@@ -14,7 +14,6 @@ three ways.
 
 References
 ----------
-
 - `Blog post on redirecting and file descriptors
   <https://eli.thegreenplace.net/2015/redirecting-all-kinds-of-stdout-in-python>`_.
 - `The capture module in pytest
@@ -23,43 +22,41 @@ References
   <https://github.com/pytest-dev/pytest/blob/main/src/_pytest/debugging.py>`_.
 
 """
+
 from __future__ import annotations
 
+import abc
 import contextlib
-import functools
 import io
 import os
 import sys
-from enum import Enum
+from io import UnsupportedOperation
 from tempfile import TemporaryFile
+from typing import TYPE_CHECKING
 from typing import Any
 from typing import AnyStr
-from typing import Generator
+from typing import BinaryIO
 from typing import Generic
-from typing import Iterator
+from typing import NamedTuple
 from typing import TextIO
+from typing import cast
+from typing import final
 
 import click
-from _pytask.config import hookimpl
-from _pytask.config_utils import parse_click_choice
-from _pytask.config_utils import ShowCapture
-from _pytask.nodes import Task
-from _pytask.shared import get_first_non_none_value
+from typing_extensions import Self
 
+from _pytask.capture_utils import CaptureMethod
+from _pytask.capture_utils import ShowCapture
+from _pytask.click import EnumChoice
+from _pytask.pluginmanager import hookimpl
+from _pytask.shared import convert_to_enum
 
-if sys.version_info >= (3, 8):
-    from typing import final as final
-else:
+if TYPE_CHECKING:
+    from collections.abc import Generator
+    from collections.abc import Iterator
+    from types import TracebackType
 
-    def final(f):
-        return f
-
-
-class _CaptureMethod(str, Enum):
-    FD = "fd"
-    NO = "no"
-    SYS = "sys"
-    TEE_SYS = "tee-sys"
+    from _pytask.node_protocols import PTask
 
 
 @hookimpl
@@ -68,55 +65,37 @@ def pytask_extend_command_line_interface(cli: click.Group) -> None:
     additional_parameters = [
         click.Option(
             ["--capture"],
-            type=click.Choice(_CaptureMethod),  # type: ignore[arg-type]
-            help="Per task capturing method. [dim]\\[default: fd][/]",
+            type=EnumChoice(CaptureMethod),
+            default=CaptureMethod.FD,
+            help="Per task capturing method.",
         ),
         click.Option(
             ["-s"],
             is_flag=True,
+            default=False,
             help="Shortcut for --capture=no.",
         ),
         click.Option(
             ["--show-capture"],
-            type=click.Choice(ShowCapture),  # type: ignore[arg-type]
-            help=(
-                "Choose which captured output should be shown for failed tasks. "
-                "[dim]\\[default: all][/]"
-            ),
+            type=EnumChoice(ShowCapture),
+            default=ShowCapture.ALL,
+            help="Choose which captured output should be shown for failed tasks.",
         ),
     ]
     cli.commands["build"].params.extend(additional_parameters)
 
 
 @hookimpl
-def pytask_parse_config(
-    config: dict[str, Any],
-    config_from_cli: dict[str, Any],
-    config_from_file: dict[str, Any],
-) -> None:
+def pytask_parse_config(config: dict[str, Any]) -> None:
     """Parse configuration.
 
     Note that, ``-s`` is a shortcut for ``--capture=no``.
 
     """
-    if config_from_cli.get("s"):
-        config["capture"] = _CaptureMethod.NO
-    else:
-        config["capture"] = get_first_non_none_value(
-            config_from_cli,
-            config_from_file,
-            key="capture",
-            default=_CaptureMethod.FD,
-            callback=parse_click_choice("capture", _CaptureMethod),
-        )
-
-    config["show_capture"] = get_first_non_none_value(
-        config_from_cli,
-        config_from_file,
-        key="show_capture",
-        default=ShowCapture.ALL,
-        callback=parse_click_choice("show_capture", ShowCapture),
-    )
+    config["capture"] = convert_to_enum(config["capture"], CaptureMethod)
+    if config["s"]:
+        config["capture"] = CaptureMethod.NO
+    config["show_capture"] = convert_to_enum(config["show_capture"], ShowCapture)
 
 
 @hookimpl
@@ -147,8 +126,10 @@ class EncodedFile(io.TextIOWrapper):
 
     @property
     def mode(self) -> str:
-        # TextIOWrapper doesn't expose a mode, but at least some of our tests check it.
-        return self.buffer.mode.replace("b", "")
+        # TextIOWrapper doesn't expose a mode, but at least some of our
+        # tests check it.
+        assert hasattr(self.buffer, "mode")
+        return cast("str", self.buffer.mode.replace("b", ""))
 
 
 class CaptureIO(io.TextIOWrapper):
@@ -170,25 +151,39 @@ class TeeCaptureIO(CaptureIO):
         return self._other.write(s)
 
 
-class DontReadFromInput:
+class DontReadFromInput(TextIO):
     """Class to disable reading from stdin while capturing is activated."""
 
-    encoding = None
+    @property
+    def encoding(self) -> str:
+        assert sys.__stdin__ is not None
+        return sys.__stdin__.encoding
 
-    def read(self, *_args: Any) -> None:  # noqa: U101
-        raise OSError(
+    def read(self, size: int = -1) -> str:  # noqa: ARG002
+        msg = (
             "pytask: reading from stdin while output is captured! Consider using `-s`."
         )
+        raise OSError(msg)
 
     readline = read
-    readlines = read
-    __next__ = read
 
-    def __iter__(self) -> DontReadFromInput:
+    def __next__(self) -> str:
+        return self.readline()
+
+    def readlines(self, hint: int | None = -1) -> list[str]:  # noqa: ARG002
+        msg = "reading from stdin while output is captured!  Consider using `-s`."
+        raise OSError(msg)
+
+    def __iter__(self) -> Iterator[str]:
         return self
 
     def fileno(self) -> int:
-        raise io.UnsupportedOperation("redirected stdin is pseudofile, has no fileno()")
+        msg = "redirected stdin is pseudofile, has no fileno()"
+        raise UnsupportedOperation(msg)
+
+    def flush(self) -> None:
+        msg = "redirected stdin is pseudofile, has no flush()"
+        raise UnsupportedOperation(msg)
 
     def isatty(self) -> bool:
         return False
@@ -196,43 +191,134 @@ class DontReadFromInput:
     def close(self) -> None:
         pass
 
-    @property
-    def buffer(self) -> DontReadFromInput:
+    def readable(self) -> bool:
+        return False
+
+    def seek(self, offset: int, whence: int = 0) -> int:  # noqa: ARG002
+        msg = "Redirected stdin is pseudofile, has no seek(int)."
+        raise UnsupportedOperation(msg)
+
+    def seekable(self) -> bool:
+        return False
+
+    def tell(self) -> int:
+        msg = "Redirected stdin is pseudofile, has no tell()."
+        raise UnsupportedOperation(msg)
+
+    def truncate(self, size: int | None = None) -> int:  # noqa: ARG002
+        msg = "Cannot truncate stdin."
+        raise UnsupportedOperation(msg)
+
+    def write(self, data: str) -> int:  # noqa: ARG002
+        msg = "Cannot write to stdin."
+        raise UnsupportedOperation(msg)
+
+    def writelines(self, *args: Any) -> None:  # noqa: ARG002
+        msg = "Cannot write to stdin."
+        raise UnsupportedOperation(msg)
+
+    def writable(self) -> bool:
+        return False
+
+    def __enter__(self) -> Self:
         return self
 
+    def __exit__(
+        self,
+        type: type[BaseException] | None,  # noqa: A002
+        value: BaseException | None,
+        traceback: TracebackType | None,
+    ) -> None:
+        pass
+
+    @property
+    def buffer(self) -> BinaryIO:
+        # The str/bytes doesn't actually matter in this type, so OK to fake.
+        return self  # type: ignore[return-value]
+
 
 # Capture classes.
 
 
+class CaptureBase(abc.ABC, Generic[AnyStr]):
+    EMPTY_BUFFER: AnyStr
+
+    @abc.abstractmethod
+    def __init__(self, fd: int) -> None:
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    def start(self) -> None:
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    def done(self) -> None:
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    def suspend(self) -> None:
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    def resume(self) -> None:
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    def writeorg(self, data: AnyStr) -> None:
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    def snap(self) -> AnyStr:
+        raise NotImplementedError
+
+
 patchsysdict = {0: "stdin", 1: "stdout", 2: "stderr"}
-"""Dict[int, str]: Map file descriptors to their names."""
+"""dict[int, str]: Map file descriptors to their names."""
 
 
-class NoCapture:
+class NoCapture(CaptureBase[str]):
     """Dummy class when capturing is disabled."""
 
-    EMPTY_BUFFER = None
-    __init__ = start = done = suspend = resume = lambda *_args: None  # noqa: U101
+    EMPTY_BUFFER = ""
+
+    def __init__(self, fd: int) -> None:
+        pass
+
+    def start(self) -> None:
+        pass
+
+    def done(self) -> None:
+        pass
+
+    def suspend(self) -> None:
+        pass
+
+    def resume(self) -> None:
+        pass
+
+    def snap(self) -> str:
+        return ""
+
+    def writeorg(self, data: str) -> None:
+        pass
 
 
-class SysCaptureBinary:
+class SysCaptureBase(CaptureBase[AnyStr]):
     """Capture IO to/from Python's buffer for stdin, stdout, and stderr.
 
     Instead of :class:`SysCapture`, this class produces bytes instead of text.
 
     """
 
-    EMPTY_BUFFER = b""
-
-    def __init__(  # type: ignore
+    def __init__(
         self,
         fd: int,
-        tmpfile=None,
+        tmpfile: TextIO | None = None,
         *,
         tee: bool = False,
     ) -> None:
         name = patchsysdict[fd]
-        self._old = getattr(sys, name)
+        self._old: TextIO = getattr(sys, name)
         self.name = name
         if tmpfile is None:
             if name == "stdin":
@@ -242,11 +328,11 @@ class SysCaptureBinary:
         self.tmpfile = tmpfile
         self._state = "initialized"
 
-    def repr(self, class_name: str) -> str:  # noqa: A003
+    def repr(self, class_name: str) -> str:
         return "<{} {} _old={} _state={!r} tmpfile={!r}>".format(
             class_name,
             self.name,
-            hasattr(self, "_old") and repr(self._old) or "<UNSET>",
+            (hasattr(self, "_old") and repr(self._old)) or "<UNSET>",
             self._state,
             self.tmpfile,
         )
@@ -255,16 +341,16 @@ class SysCaptureBinary:
         return "<{} {} _old={} _state={!r} tmpfile={!r}>".format(
             self.__class__.__name__,
             self.name,
-            hasattr(self, "_old") and repr(self._old) or "<UNSET>",
+            (hasattr(self, "_old") and repr(self._old)) or "<UNSET>",
             self._state,
             self.tmpfile,
         )
 
     def _assert_state(self, op: str, states: tuple[str, ...]) -> None:
-        assert (
-            self._state in states
-        ), "cannot {} in state {!r}: expected one of {}".format(
-            op, self._state, ", ".join(states)
+        assert self._state in states, (
+            "cannot {} in state {!r}: expected one of {}".format(
+                op, self._state, ", ".join(states)
+            )
         )
 
     def start(self) -> None:
@@ -272,14 +358,6 @@ class SysCaptureBinary:
         setattr(sys, self.name, self.tmpfile)
         self._state = "started"
 
-    def snap(self) -> str:
-        self._assert_state("snap", ("started", "suspended"))
-        self.tmpfile.seek(0)
-        res = self.tmpfile.buffer.read()
-        self.tmpfile.seek(0)
-        self.tmpfile.truncate()
-        return res
-
     def done(self) -> None:
         self._assert_state("done", ("initialized", "started", "suspended", "done"))
         if self._state == "done":
@@ -301,23 +379,37 @@ class SysCaptureBinary:
         setattr(sys, self.name, self.tmpfile)
         self._state = "started"
 
-    def writeorg(self, data: str) -> None:
+
+class SysCaptureBinary(SysCaptureBase[bytes]):
+    EMPTY_BUFFER = b""
+
+    def snap(self) -> bytes:
+        self._assert_state("snap", ("started", "suspended"))
+        self.tmpfile.seek(0)
+        res = self.tmpfile.buffer.read()
+        self.tmpfile.seek(0)
+        self.tmpfile.truncate()
+        return res
+
+    def writeorg(self, data: bytes) -> None:
         self._assert_state("writeorg", ("started", "suspended"))
         self._old.flush()
         self._old.buffer.write(data)
         self._old.buffer.flush()
 
 
-class SysCapture(SysCaptureBinary):
+class SysCapture(SysCaptureBase[str]):
     """Capture IO to/from Python's buffer for stdin, stdout, and stderr.
 
     Instead of :class:`SysCaptureBinary`, this class produces text instead of bytes.
 
     """
 
-    EMPTY_BUFFER = ""  # type: ignore[assignment]
+    EMPTY_BUFFER = ""
 
     def snap(self) -> str:
+        self._assert_state("snap", ("started", "suspended"))
+        assert isinstance(self.tmpfile, CaptureIO)
         res = self.tmpfile.getvalue()
         self.tmpfile.seek(0)
         self.tmpfile.truncate()
@@ -329,15 +421,13 @@ class SysCapture(SysCaptureBinary):
         self._old.flush()
 
 
-class FDCaptureBinary:
+class FDCaptureBase(CaptureBase[AnyStr]):
     """Capture IO to/from a given OS-level file descriptor.
 
     snap() produces `bytes`.
 
     """
 
-    EMPTY_BUFFER = b""
-
     def __init__(self, targetfd: int) -> None:
         self.targetfd = targetfd
 
@@ -362,11 +452,11 @@ class FDCaptureBinary:
         self.targetfd_save = os.dup(targetfd)
 
         if targetfd == 0:
-            self.tmpfile = open(os.devnull)
-            self.syscapture = SysCapture(targetfd)
+            self.tmpfile = open(os.devnull, encoding="utf-8")  # noqa: SIM115, PTH123
+            self.syscapture: CaptureBase[str] = SysCapture(targetfd)
         else:
             self.tmpfile = EncodedFile(
-                TemporaryFile(buffering=0),
+                TemporaryFile(buffering=0),  # noqa: SIM115
                 encoding="utf-8",
                 errors="replace",
                 newline="",
@@ -375,12 +465,12 @@ class FDCaptureBinary:
             if targetfd in patchsysdict:
                 self.syscapture = SysCapture(targetfd, self.tmpfile)
             else:
-                self.syscapture = NoCapture()
+                self.syscapture = NoCapture(targetfd)
 
         self._state = "initialized"
 
     def __repr__(self) -> str:
-        return "<{} {} oldfd={} _state={!r} tmpfile={!r}>".format(
+        return "<{} {} oldfd={} _state={!r} tmpfile={!r}>".format(  # noqa: UP032
             self.__class__.__name__,
             self.targetfd,
             self.targetfd_save,
@@ -389,10 +479,10 @@ class FDCaptureBinary:
         )
 
     def _assert_state(self, op: str, states: tuple[str, ...]) -> None:
-        assert (
-            self._state in states
-        ), "cannot {} in state {!r}: expected one of {}".format(
-            op, self._state, ", ".join(states)
+        assert self._state in states, (
+            "cannot {} in state {!r}: expected one of {}".format(
+                op, self._state, ", ".join(states)
+            )
         )
 
     def start(self) -> None:
@@ -402,17 +492,13 @@ class FDCaptureBinary:
         self.syscapture.start()
         self._state = "started"
 
-    def snap(self) -> bytes:
-        self._assert_state("snap", ("started", "suspended"))
-        self.tmpfile.seek(0)
-        res = self.tmpfile.buffer.read()
-        self.tmpfile.seek(0)
-        self.tmpfile.truncate()
-        return res
-
     def done(self) -> None:
-        """Stop capturing, restore streams, return original capture file, seeked to
-        position zero."""
+        """Stop capturing.
+
+        Stop capturing, restore streams, return original capture file, sought to
+        position zero.
+
+        """
         self._assert_state("done", ("initialized", "started", "suspended", "done"))
         if self._state == "done":
             return
@@ -442,24 +528,39 @@ class FDCaptureBinary:
         os.dup2(self.tmpfile.fileno(), self.targetfd)
         self._state = "started"
 
+
+class FDCaptureBinary(FDCaptureBase[bytes]):
+    """Capture IO to/from a given OS-level file descriptor.
+
+    snap() produces `bytes`.
+    """
+
+    EMPTY_BUFFER = b""
+
+    def snap(self) -> bytes:
+        self._assert_state("snap", ("started", "suspended"))
+        self.tmpfile.seek(0)
+        res = self.tmpfile.buffer.read()
+        self.tmpfile.seek(0)
+        self.tmpfile.truncate()
+        return res  # type: ignore[return-value]
+
     def writeorg(self, data: bytes) -> None:
         """Write to original file descriptor."""
         self._assert_state("writeorg", ("started", "suspended"))
         os.write(self.targetfd_save, data)
 
 
-class FDCapture(FDCaptureBinary):
+class FDCapture(FDCaptureBase[str]):
     """Capture IO to/from a given OS-level file descriptor.
 
     snap() produces text.
 
     """
 
-    # Ignore type because it doesn't match the type in the superclass (bytes).
-    EMPTY_BUFFER = ""  # type: ignore
+    EMPTY_BUFFER = ""
 
-    # Ignore type because it doesn't match the type in the superclass (bytes).
-    def snap(self) -> str:  # type: ignore
+    def snap(self) -> str:
         self._assert_state("snap", ("started", "suspended"))
         self.tmpfile.seek(0)
         res = self.tmpfile.read()
@@ -467,72 +568,36 @@ class FDCapture(FDCaptureBinary):
         self.tmpfile.truncate()
         return res
 
-    # Ignore type because it doesn't match the type in the superclass (bytes).
-    def writeorg(self, data: str) -> None:  # type: ignore
+    def writeorg(self, data: str) -> None:
         """Write to original file descriptor."""
-        super().writeorg(data.encode("utf-8"))
+        self._assert_state("writeorg", ("started", "suspended"))
+        # Will be fixed by pytest. Use encoding of original stream
+        os.write(self.targetfd_save, data.encode("utf-8"))
 
 
 # MultiCapture
 
 
-@final
-@functools.total_ordering
-class CaptureResult(Generic[AnyStr]):
-    """The result of :meth:`MultiCapture.readouterr` which wraps stdout and stderr.
+# Generic NamedTuple only supported since Python 3.11.
+if sys.version_info >= (3, 11) or TYPE_CHECKING:
 
-    This class was a namedtuple, but due to mypy limitation [0]_ it could not be made
-    generic, so was replaced by a regular class which tries to emulate the pertinent
-    parts of a namedtuple. If the mypy limitation is ever lifted, can make it a
-    namedtuple again.
+    @final
+    class CaptureResult(NamedTuple, Generic[AnyStr]):
+        """A class for capture results."""
 
-    .. [0] https://github.com/python/mypy/issues/685
+        out: AnyStr
+        err: AnyStr
 
-    """
+else:
+    import collections
 
-    __slots__ = ("out", "err")
+    class CaptureResult(
+        collections.namedtuple("CaptureResult", ["out", "err"]),  # noqa: PYI024
+        Generic[AnyStr],
+    ):
+        """A class for capture results."""
 
-    def __init__(self, out: AnyStr, err: AnyStr) -> None:
-        self.out: AnyStr = out
-        self.err: AnyStr = err
-
-    def __len__(self) -> int:
-        return 2
-
-    def __iter__(self) -> Iterator[AnyStr]:
-        return iter((self.out, self.err))
-
-    def __getitem__(self, item: int) -> AnyStr:
-        return tuple(self)[item]
-
-    def _replace(
-        self, *, out: AnyStr | None = None, err: AnyStr | None = None
-    ) -> CaptureResult[AnyStr]:
-        return CaptureResult(
-            out=self.out if out is None else out, err=self.err if err is None else err
-        )
-
-    def count(self, value: AnyStr) -> int:
-        return tuple(self).count(value)
-
-    def index(self, value: int) -> int:
-        return tuple(self).index(value)
-
-    def __eq__(self, other: object) -> bool:
-        if not isinstance(other, (CaptureResult, tuple)):
-            return NotImplemented
-        return tuple(self) == tuple(other)
-
-    def __hash__(self) -> int:
-        return hash(tuple(self))
-
-    def __lt__(self, other: object) -> bool:
-        if not isinstance(other, (CaptureResult, tuple)):
-            return NotImplemented
-        return tuple(self) < tuple(other)
-
-    def __repr__(self) -> str:
-        return f"CaptureResult(out={self.out!r}, err={self.err!r})"
+        __slots__ = ()
 
 
 class MultiCapture(Generic[AnyStr]):
@@ -549,18 +614,17 @@ class MultiCapture(Generic[AnyStr]):
 
     def __init__(
         self,
-        in_: FDCapture | SysCapture | None,
-        out: FDCapture | SysCapture | None,
-        err: FDCapture | SysCapture | None,
+        in_: CaptureBase[AnyStr] | None,
+        out: CaptureBase[AnyStr] | None,
+        err: CaptureBase[AnyStr] | None,
     ) -> None:
-        self.in_ = in_
-        self.out = out
-        self.err = err
+        self.in_: CaptureBase[AnyStr] | None = in_
+        self.out: CaptureBase[AnyStr] | None = out
+        self.err: CaptureBase[AnyStr] | None = err
 
     def __repr__(self) -> str:
-        return (
-            "<MultiCapture out={!r} err={!r} in_={!r} _state={!r} "
-            "_in_suspended={!r}>"
+        return (  # noqa: UP032
+            "<MultiCapture out={!r} err={!r} in_={!r} _state={!r} _in_suspended={!r}>"
         ).format(
             self.out,
             self.err,
@@ -582,9 +646,9 @@ class MultiCapture(Generic[AnyStr]):
         """Pop current snapshot out/err capture and flush to orig streams."""
         out, err = self.readouterr()
         if out:
-            self.out.writeorg(out)  # type: ignore
+            self.out.writeorg(out)  # type: ignore[union-attr]
         if err:
-            self.err.writeorg(err)  # type: ignore
+            self.err.writeorg(err)  # type: ignore[union-attr]
         return out, err
 
     def suspend_capturing(self, in_: bool = False) -> None:
@@ -604,13 +668,14 @@ class MultiCapture(Generic[AnyStr]):
         if self.err:
             self.err.resume()
         if self._in_suspended:
-            self.in_.resume()
+            self.in_.resume()  # type: ignore[union-attr]
             self._in_suspended = False
 
     def stop_capturing(self) -> None:
         """Stop capturing and reset capturing streams."""
         if self._state == "stopped":
-            raise ValueError("was already stopped")
+            msg = "was already stopped"
+            raise ValueError(msg)
         self._state = "stopped"
         if self.out:
             self.out.done()
@@ -624,35 +689,31 @@ class MultiCapture(Generic[AnyStr]):
         return self._state == "started"
 
     def readouterr(self) -> CaptureResult[AnyStr]:
-        if self.out:
-            out = self.out.snap()
-        else:
-            out = ""
-        if self.err:
-            err = self.err.snap()
-        else:
-            err = ""
-        return CaptureResult(out, err)  # type: ignore
+        out = self.out.snap() if self.out else ""
+        err = self.err.snap() if self.err else ""
+        # Will be fixed by pytest. This type error is real, need to fix.
+        return CaptureResult(out, err)  # type: ignore[arg-type]
 
 
-def _get_multicapture(method: _CaptureMethod) -> MultiCapture[str]:
+def _get_multicapture(method: CaptureMethod) -> MultiCapture[str]:
     """Set up the MultiCapture class with the passed method.
 
     For each valid method, the function instantiates the :class:`MultiCapture` class
     with the specified buffers for ``stdin``, ``stdout``, and ``stderr``.
 
     """
-    if method == _CaptureMethod.FD:
+    if method == CaptureMethod.FD:
         return MultiCapture(in_=FDCapture(0), out=FDCapture(1), err=FDCapture(2))
-    elif method == _CaptureMethod.SYS:
+    if method == CaptureMethod.SYS:
         return MultiCapture(in_=SysCapture(0), out=SysCapture(1), err=SysCapture(2))
-    elif method == _CaptureMethod.NO:
+    if method == CaptureMethod.NO:
         return MultiCapture(in_=None, out=None, err=None)
-    elif method == _CaptureMethod.TEE_SYS:
+    if method == CaptureMethod.TEE_SYS:
         return MultiCapture(
             in_=None, out=SysCapture(1, tee=True), err=SysCapture(2, tee=True)
         )
-    raise ValueError(f"unknown capturing method: {method!r}")
+    msg = f"unknown capturing method: {method!r}"
+    raise ValueError(msg)
 
 
 # Own implementation of the CaptureManager.
@@ -671,12 +732,12 @@ class CaptureManager:
 
     """
 
-    def __init__(self, method: _CaptureMethod) -> None:
+    def __init__(self, method: CaptureMethod) -> None:
         self._method = method
         self._capturing: MultiCapture[str] | None = None
 
     def __repr__(self) -> str:
-        return ("<CaptureManager _method={!r} _capturing={!r}>").format(
+        return ("<CaptureManager _method={!r} _capturing={!r}>").format(  # noqa: UP032
             self._method, self._capturing
         )
 
@@ -711,7 +772,7 @@ class CaptureManager:
     # Helper context managers
 
     @contextlib.contextmanager
-    def task_capture(self, when: str, task: Task) -> Generator[None, None, None]:
+    def task_capture(self, when: str, task: PTask) -> Generator[None, None, None]:
         """Pipe captured stdout and stderr into report sections."""
         self.resume()
 
@@ -720,31 +781,33 @@ class CaptureManager:
         finally:
             self.suspend(in_=False)
 
-        out, err = self.read()
-        task.add_report_section(when, "stdout", out)
-        task.add_report_section(when, "stderr", err)
+            out, err = self.read()
+            if out:
+                task.report_sections.append((when, "stdout", out))
+            if err:
+                task.report_sections.append((when, "stderr", err))
 
     # Hooks
 
-    @hookimpl(hookwrapper=True)
-    def pytask_execute_task_setup(self, task: Task) -> Generator[None, None, None]:
+    @hookimpl(wrapper=True)
+    def pytask_execute_task_setup(self, task: PTask) -> Generator[None, None, None]:
         """Capture output during setup."""
         with self.task_capture("setup", task):
-            yield
+            return (yield)
 
-    @hookimpl(hookwrapper=True)
-    def pytask_execute_task(self, task: Task) -> Generator[None, None, None]:
+    @hookimpl(wrapper=True)
+    def pytask_execute_task(self, task: PTask) -> Generator[None, None, None]:
         """Capture output during execution."""
         with self.task_capture("call", task):
-            yield
+            return (yield)
 
-    @hookimpl(hookwrapper=True)
-    def pytask_execute_task_teardown(self, task: Task) -> Generator[None, None, None]:
+    @hookimpl(wrapper=True)
+    def pytask_execute_task_teardown(self, task: PTask) -> Generator[None, None, None]:
         """Capture output during teardown."""
         with self.task_capture("teardown", task):
-            yield
+            return (yield)
 
-    @hookimpl(hookwrapper=True)
+    @hookimpl(wrapper=True)
     def pytask_collect_log(self) -> Generator[None, None, None]:
         """Suspend capturing at the end of the collection.
 
@@ -757,4 +820,4 @@ class CaptureManager:
 
         """
         self.suspend(in_=True)
-        yield
+        return (yield)
