@@ -1,14 +1,18 @@
-"""This module contains functions to assess compatibility and optional dependencies."""
+"""Contains functions to assess compatibility and optional dependencies."""
+
 from __future__ import annotations
 
-import importlib
 import shutil
 import sys
-import types
 import warnings
+from importlib import import_module
+from typing import TYPE_CHECKING
 
 from packaging.version import parse as parse_version
 
+if TYPE_CHECKING:
+    import types
+
 
 __all__ = ["check_for_optional_program", "import_optional_dependency"]
 
@@ -26,7 +30,8 @@ def _get_version(module: types.ModuleType) -> str:
     """Get version from a package."""
     version = getattr(module, "__version__", None)
     if version is None:
-        raise ImportError(f"Can't determine version for {module.__name__}")
+        msg = f"Can't determine version for {module.__name__}"
+        raise ImportError(msg)
     return version
 
 
@@ -44,11 +49,11 @@ def import_optional_dependency(
 
     Parameters
     ----------
-    name : str
+    name
         The module name.
-    extra : str
+    extra
         Additional text to include in the ImportError message.
-    errors : str {'raise', 'warn', 'ignore'}
+    errors
         What to do when a dependency is not found or its version is too old.
 
         * raise : Raise an ImportError
@@ -57,22 +62,23 @@ def import_optional_dependency(
         * ignore: If the module is not installed, return None, otherwise, return the
           module, even if the version is too old. It's expected that users validate the
           version locally when using ``errors="ignore"`` (see. ``io/html.py``)
-    min_version : str, default None
+    min_version
         Specify a minimum version that is different from the global pandas minimum
         version required.
-    caller : str, default "pytask"
+    caller
         The caller of the function.
 
     Returns
     -------
-    maybe_module : Optional[ModuleType]
+    types.ModuleType | None
         The imported module, when found and the version is correct. None is returned
         when the package is not found and `errors` is False, or when the package's
         version is too old and `errors` is ``'warn'``.
 
     """
     if errors not in ("warn", "raise", "ignore"):  # pragma: no cover
-        raise ValueError("'errors' must be one of 'warn', 'raise' or 'ignore'.")
+        msg = "'errors' must be one of 'warn', 'raise' or 'ignore'."
+        raise ValueError(msg)
 
     package_name = _IMPORT_TO_PACKAGE_NAME.get(name)
     install_name = package_name if package_name is not None else name
@@ -84,12 +90,13 @@ def import_optional_dependency(
         f"Use pip or conda to install {install_name!r}."
     )
     try:
-        module = importlib.import_module(name)
+        # The from import is used to avoid monkeypatching errors in some tests. See
+        # https://stackoverflow.com/a/31746577 for more information.
+        module = import_module(name)
     except ImportError:
         if errors == "raise":
             raise ImportError(msg) from None
-        else:
-            return None
+        return None
 
     # Handle submodules: if we have submodule, grab parent module from sys.modules
     parent = name.split(".")[0]
@@ -109,9 +116,9 @@ def import_optional_dependency(
                 f"{parent!r} (version {version!r} currently installed)."
             )
             if errors == "warn":
-                warnings.warn(msg, UserWarning)
+                warnings.warn(msg, UserWarning, stacklevel=2)
                 return None
-            elif errors == "raise":
+            if errors == "raise":
                 raise ImportError(msg)
 
     return module
@@ -125,9 +132,8 @@ def check_for_optional_program(
 ) -> bool | None:
     """Check whether an optional program exists."""
     if errors not in ("warn", "raise", "ignore"):
-        raise ValueError(
-            f"'errors' must be one of 'warn', 'raise' or 'ignore' and not {errors!r}."
-        )
+        msg = f"'errors' must be one of 'warn', 'raise' or 'ignore' and not {errors!r}."
+        raise ValueError(msg)
 
     msg = f"{caller} requires the optional program {name!r}. {extra}"
 
@@ -136,7 +142,7 @@ def check_for_optional_program(
     if not program_exists:
         if errors == "raise":
             raise RuntimeError(msg)
-        elif errors == "warn":
-            warnings.warn(msg, UserWarning)
+        if errors == "warn":
+            warnings.warn(msg, UserWarning, stacklevel=2)
 
     return program_exists
