@@ -1,4 +1,5 @@
-"""This module contains custom exceptions."""
+"""Contains custom exceptions."""
+
 from __future__ import annotations
 
 
@@ -14,6 +15,10 @@ class NodeNotCollectedError(PytaskError):
     """Exception for nodes which could not be collected."""
 
 
+class NodeLoadError(PytaskError):
+    """Exception for nodes whose value could not be loaded."""
+
+
 class ConfigurationError(PytaskError):
     """Exception during the configuration."""
 
