@@ -4,34 +4,36 @@ At each of the entry-points, a plugin can register a hook implementation which r
 the message send by the host and may send a response.
 
 """
+
 from __future__ import annotations
 
-from pathlib import Path
-from typing import Any
-from typing import Callable
 from typing import TYPE_CHECKING
+from typing import Any
 
-import click
-import networkx as nx
 import pluggy
 
-
 if TYPE_CHECKING:
-    from _pytask.session import Session
-    from _pytask.nodes import MetaNode
-    from _pytask.nodes import Task
+    from pathlib import Path
+
+    import click
+    from pluggy import PluginManager
+
+    from _pytask.models import NodeInfo
+    from _pytask.node_protocols import PNode
+    from _pytask.node_protocols import PProvisionalNode
+    from _pytask.node_protocols import PTask
     from _pytask.outcomes import CollectionOutcome
     from _pytask.outcomes import TaskOutcome
     from _pytask.reports import CollectionReport
     from _pytask.reports import ExecutionReport
-    from _pytask.reports import ResolveDependencyReport
+    from _pytask.session import Session
 
 
 hookspec = pluggy.HookspecMarker("pytask")
 
 
-@hookspec
-def pytask_add_hooks(pm: pluggy.PluginManager) -> None:
+@hookspec(historic=True)
+def pytask_add_hooks(pm: PluginManager) -> None:
     """Add hook specifications and implementations to the plugin manager.
 
     This hook is the first to be called to let plugins register their hook
@@ -46,7 +48,7 @@ def pytask_add_hooks(pm: pluggy.PluginManager) -> None:
 # Hooks for the command-line interface.
 
 
-@hookspec
+@hookspec(historic=True)
 def pytask_extend_command_line_interface(cli: click.Group) -> None:
     """Extend the command line interface.
 
@@ -65,9 +67,7 @@ def pytask_extend_command_line_interface(cli: click.Group) -> None:
 
 
 @hookspec(firstresult=True)
-def pytask_configure(
-    pm: pluggy.PluginManager, config_from_cli: dict[str, Any]
-) -> dict[str, Any]:
+def pytask_configure(pm: PluginManager, raw_config: dict[str, Any]) -> dict[str, Any]:
     """Configure pytask.
 
     The main hook implementation which controls the configuration and calls subordinated
@@ -77,20 +77,8 @@ def pytask_configure(
 
 
 @hookspec
-def pytask_parse_config(
-    config: dict[str, Any],
-    config_from_cli: dict[str, Any],
-    config_from_file: dict[str, Any],
-) -> None:
-    """Parse configuration from the CLI or from file.
-
-    This hook can be used to unify the configuration from the command line interface,
-    the configuration file and provided defaults. The function
-    :func:`pytask.shared.get_first_non_none_value` might be helpful for that.
-
-    Note that, the configuration is changed in-place.
-
-    """
+def pytask_parse_config(config: dict[str, Any]) -> None:
+    """Parse configuration that is from CLI or file."""
 
 
 @hookspec
@@ -139,7 +127,7 @@ def pytask_ignore_collect(path: Path, config: dict[str, Any]) -> bool:
 
 
 @hookspec
-def pytask_collect_modify_tasks(session: Session, tasks: list[Task]) -> None:
+def pytask_collect_modify_tasks(session: Session, tasks: list[PTask]) -> None:
     """Modify tasks after they have been collected.
 
     This hook can be used to deselect tasks when they match a certain keyword or mark.
@@ -177,25 +165,27 @@ def pytask_collect_file_log(session: Session, reports: list[CollectionReport]) -
 
 @hookspec(firstresult=True)
 def pytask_collect_task_protocol(
-    session: Session, path: Path, name: str, obj: Any
+    session: Session, path: Path | None, name: str, obj: Any
 ) -> CollectionReport | None:
     """Start protocol to collect tasks."""
 
 
 @hookspec
 def pytask_collect_task_setup(
-    session: Session, path: Path, name: str, obj: Any
+    session: Session, path: Path | None, name: str, obj: Any
 ) -> None:
-    """Steps before collecting a task."""
+    """Set up collecting a task."""
 
 
 @hookspec(firstresult=True)
-def pytask_collect_task(session: Session, path: Path, name: str, obj: Any) -> Task:
+def pytask_collect_task(
+    session: Session, path: Path | None, name: str, obj: Any
+) -> PTask:
     """Collect a single task."""
 
 
 @hookspec
-def pytask_collect_task_teardown(session: Session, task: Task) -> None:
+def pytask_collect_task_teardown(session: Session, task: PTask) -> None:
     """Perform tear-down operations when a task was collected.
 
     Use this hook specification to, for example, perform checks on the collected task.
@@ -205,14 +195,14 @@ def pytask_collect_task_teardown(session: Session, task: Task) -> None:
 
 @hookspec(firstresult=True)
 def pytask_collect_node(
-    session: Session, path: Path, node: MetaNode
-) -> MetaNode | None:
+    session: Session, path: Path, node_info: NodeInfo
+) -> PNode | PProvisionalNode | None:
     """Collect a node which is a dependency or a product of a task."""
 
 
 @hookspec(firstresult=True)
 def pytask_collect_log(
-    session: Session, reports: list[CollectionReport], tasks: list[Task]
+    session: Session, reports: list[CollectionReport], tasks: list[PTask]
 ) -> None:
     """Log errors occurring during the collection.
 
@@ -221,91 +211,6 @@ def pytask_collect_log(
     """
 
 
-# Hooks to parametrize tasks.
-
-
-@hookspec(firstresult=True)
-def pytask_parametrize_task(
-    session: Session, name: str, obj: Any
-) -> list[tuple[str, Callable[..., Any]]]:
-    """Generate multiple tasks from name and object with parametrization."""
-
-
-@hookspec
-def pytask_parametrize_kwarg_to_marker(obj: Any, kwargs: dict[Any, Any]) -> None:
-    """Add some keyword arguments as markers to object.
-
-    This hook moves arguments defined in the parametrization to marks of the same
-    function. This allows an argument like ``depends_on`` be transformed to the usual
-    ``@pytask.mark.depends_on`` marker which receives special treatment.
-
-    """
-
-
-# Hooks for resolving dependencies.
-
-
-@hookspec(firstresult=True)
-def pytask_resolve_dependencies(session: Session) -> None:
-    """Resolve dependencies.
-
-    The main hook implementation which controls the resolution of dependencies and calls
-    subordinated hooks.
-
-    """
-
-
-@hookspec(firstresult=True)
-def pytask_resolve_dependencies_create_dag(
-    session: Session, tasks: list[Task]
-) -> nx.DiGraph:
-    """Create the DAG.
-
-    This hook creates the DAG from tasks, dependencies and products. The DAG can be used
-    by a scheduler to find an execution order.
-
-    """
-
-
-@hookspec
-def pytask_resolve_dependencies_modify_dag(session: Session, dag: nx.DiGraph) -> None:
-    """Modify the DAG.
-
-    This hook allows to make some changes to the DAG before it is validated and tasks
-    are selected.
-
-    """
-
-
-@hookspec(firstresult=True)
-def pytask_resolve_dependencies_validate_dag(session: Session, dag: nx.DiGraph) -> None:
-    """Validate the DAG.
-
-    This hook validates the DAG. For example, there can be cycles in the DAG if tasks,
-    dependencies and products have been misspecified.
-
-    """
-
-
-@hookspec
-def pytask_resolve_dependencies_select_execution_dag(
-    session: Session, dag: nx.DiGraph
-) -> None:
-    """Select the subgraph which needs to be executed.
-
-    This hook determines which of the tasks have to be re-run because something has
-    changed.
-
-    """
-
-
-@hookspec
-def pytask_resolve_dependencies_log(
-    session: Session, report: ResolveDependencyReport
-) -> None:
-    """Log errors during resolving dependencies."""
-
-
 # Hooks for running tasks.
 
 
@@ -328,16 +233,6 @@ def pytask_execute_log_start(session: Session) -> None:
     """
 
 
-@hookspec(firstresult=True)
-def pytask_execute_create_scheduler(session: Session) -> Any:
-    """Create a scheduler for the execution.
-
-    The scheduler provides information on which tasks are able to be executed. Its
-    foundation is likely a topological ordering of the tasks based on the DAG.
-
-    """
-
-
 @hookspec(firstresult=True)
 def pytask_execute_build(session: Session) -> Any:
     """Execute the build.
@@ -348,7 +243,7 @@ def pytask_execute_build(session: Session) -> Any:
 
 
 @hookspec(firstresult=True)
-def pytask_execute_task_protocol(session: Session, task: Task) -> ExecutionReport:
+def pytask_execute_task_protocol(session: Session, task: PTask) -> ExecutionReport:
     """Run the protocol for executing a test.
 
     This hook runs all stages of the execution process, setup, execution, and teardown
@@ -360,7 +255,7 @@ def pytask_execute_task_protocol(session: Session, task: Task) -> ExecutionRepor
 
 
 @hookspec(firstresult=True)
-def pytask_execute_task_log_start(session: Session, task: Task) -> None:
+def pytask_execute_task_log_start(session: Session, task: PTask) -> None:
     """Start logging of task execution.
 
     This hook can be used to provide more verbose output during the execution.
@@ -369,7 +264,7 @@ def pytask_execute_task_log_start(session: Session, task: Task) -> None:
 
 
 @hookspec
-def pytask_execute_task_setup(session: Session, task: Task) -> None:
+def pytask_execute_task_setup(session: Session, task: PTask) -> None:
     """Set up the task execution.
 
     This hook is called before the task is executed and can provide an entry-point to
@@ -380,12 +275,12 @@ def pytask_execute_task_setup(session: Session, task: Task) -> None:
 
 
 @hookspec(firstresult=True)
-def pytask_execute_task(session: Session, task: Task) -> Any:
+def pytask_execute_task(session: Session, task: PTask) -> Any:
     """Execute a task."""
 
 
 @hookspec
-def pytask_execute_task_teardown(session: Session, task: Task) -> None:
+def pytask_execute_task_teardown(session: Session, task: PTask) -> None:
     """Tear down task execution.
 
     This hook is executed after the task has been executed. It allows to perform clean-
@@ -441,7 +336,7 @@ def pytask_log_session_footer(
 
 @hookspec
 def pytask_profile_add_info_on_task(
-    session: Session, tasks: list[Task], profile: dict[str, dict[Any, Any]]
+    session: Session, tasks: list[PTask], profile: dict[str, dict[Any, Any]]
 ) -> None:
     """Add information on task for profile.
 
