@@ -1,31 +1,39 @@
-"""This module contains code related to live objects."""
+"""Contains code related to live objects."""
+
 from __future__ import annotations
 
+from dataclasses import dataclass
+from typing import TYPE_CHECKING
 from typing import Any
-from typing import Dict
-from typing import Generator
-from typing import List
-from typing import Union
+from typing import NamedTuple
 
-import attr
 import click
-from _pytask.config import hookimpl
-from _pytask.console import console
-from _pytask.console import format_task_id
-from _pytask.nodes import Task
-from _pytask.outcomes import CollectionOutcome
-from _pytask.outcomes import TaskOutcome
-from _pytask.report import CollectionReport
-from _pytask.report import ExecutionReport
-from _pytask.session import Session
-from _pytask.shared import get_first_non_none_value
+from attrs import define
+from attrs import field
 from rich.box import ROUNDED
+from rich.errors import LiveError
 from rich.live import Live
 from rich.status import Status
 from rich.style import Style
 from rich.table import Table
 from rich.text import Text
 
+from _pytask.console import console
+from _pytask.console import format_task_name
+from _pytask.logging_utils import TaskExecutionStatus
+from _pytask.outcomes import CollectionOutcome
+from _pytask.outcomes import Exit
+from _pytask.outcomes import TaskOutcome
+from _pytask.pluginmanager import hookimpl
+
+if TYPE_CHECKING:
+    from collections.abc import Generator
+
+    from _pytask.node_protocols import PTask
+    from _pytask.reports import CollectionReport
+    from _pytask.reports import ExecutionReport
+    from _pytask.session import Session
+
 
 @hookimpl
 def pytask_extend_command_line_interface(cli: click.Group) -> None:
@@ -33,47 +41,21 @@ def pytask_extend_command_line_interface(cli: click.Group) -> None:
     additional_parameters = [
         click.Option(
             ["--n-entries-in-table"],
-            default=None,
+            default=15,
+            type=click.IntRange(min=0),
             help="How many entries to display in the table during the execution. "
-            "Tasks which are running are always displayed. [dim]\\[default: 15][/]",
+            "Tasks which are running are always displayed.",
+        ),
+        click.Option(
+            ["--sort-table/--do-not-sort-table"],
+            default=True,
+            type=bool,
+            help="Sort the table of tasks at the end of the execution.",
         ),
     ]
     cli.commands["build"].params.extend(additional_parameters)
 
 
-@hookimpl
-def pytask_parse_config(
-    config: dict[str, Any],
-    config_from_cli: dict[str, Any],
-    config_from_file: dict[str, Any],
-) -> None:
-    """Parse the configuration."""
-    config["n_entries_in_table"] = get_first_non_none_value(
-        config_from_cli,
-        config_from_file,
-        key="n_entries_in_table",
-        default=15,
-        callback=_parse_n_entries_in_table,
-    )
-
-
-def _parse_n_entries_in_table(value: int | str | None) -> int:
-    """Parse how many entries should be displayed in the table during the execution."""
-    if value in ("none", "None", None, ""):
-        out = None
-    elif isinstance(value, int) and value >= 1:
-        out = value
-    elif isinstance(value, str) and value.isdigit() and int(value) >= 1:
-        out = int(value)
-    elif value == "all":
-        out = 1_000_000
-    else:
-        raise ValueError(
-            "'n_entries_in_table' can either be 'None' or an integer bigger than one."
-        )
-    return out
-
-
 @hookimpl
 def pytask_post_parse(config: dict[str, Any]) -> None:
     """Post-parse the configuration."""
@@ -94,17 +76,16 @@ def pytask_post_parse(config: dict[str, Any]) -> None:
     config["pm"].register(live_collection, "live_collection")
 
 
-@hookimpl(hookwrapper=True)
+@hookimpl(wrapper=True)
 def pytask_execute(session: Session) -> Generator[None, None, None]:
     if session.config["verbose"] >= 1:
         live_execution = session.config["pm"].get_plugin("live_execution")
         if live_execution:
             live_execution.n_tasks = len(session.tasks)
-
-    yield
+    return (yield)
 
 
-@attr.s(eq=False)
+@define(eq=False)
 class LiveManager:
     """A class for live displays during a session.
 
@@ -124,10 +105,21 @@ class LiveManager:
 
     """
 
-    _live = Live(renderable=None, console=console, auto_refresh=False)
+    _live: Live = field(
+        factory=lambda: Live(renderable=None, console=console, auto_refresh=False)
+    )
 
     def start(self) -> None:
-        self._live.start()
+        try:
+            self._live.start()
+        except LiveError:
+            msg = (
+                "pytask tried to launch a second live display which is impossible. The "
+                "issue occurs when you use pytask on the command line on a task module "
+                "that uses the programmatic interface of pytask at the same time. "
+                "Use either the command line or the programmatic interface."
+            )
+            raise Exit(msg) from None
 
     def stop(self, transient: bool | None = None) -> None:
         if transient is not None:
@@ -149,46 +141,60 @@ class LiveManager:
         self._live.refresh()
 
     @property
-    def is_started(self) -> None:
+    def is_started(self) -> bool:
         return self._live.is_started
 
 
-@attr.s(eq=False, kw_only=True)
+@dataclass
+class _TaskEntry:
+    task: PTask
+    status: TaskExecutionStatus
+
+
+class _ReportEntry(NamedTuple):
+    name: str
+    outcome: TaskOutcome
+    task: PTask
+
+
+@define(eq=False, kw_only=True)
 class LiveExecution:
     """A class for managing the table displaying task progress during the execution."""
 
-    live_manager = attr.ib(type=LiveManager)
-    n_entries_in_table = attr.ib(type=int)
-    verbose = attr.ib(type=int)
-    editor_url_scheme = attr.ib(type=str)
-    sort_final_table = attr.ib(default=False, type=bool)
-    n_tasks = attr.ib(default="x", type=Union[int, str])
-    _reports = attr.ib(factory=list, type=List[Dict[str, Any]])
-    _running_tasks = attr.ib(factory=dict, type=Dict[str, Task])
+    live_manager: LiveManager
+    n_entries_in_table: int
+    verbose: int
+    editor_url_scheme: str
+    initial_status: TaskExecutionStatus = TaskExecutionStatus.RUNNING
+    sort_final_table: bool = False
+    n_tasks: int | str = "x"
+    _reports: list[_ReportEntry] = field(factory=list)
+    _running_tasks: dict[str, _TaskEntry] = field(factory=dict)
 
-    @hookimpl(hookwrapper=True)
+    @hookimpl(wrapper=True)
     def pytask_execute_build(self) -> Generator[None, None, None]:
-        """Wrap the execution with the live manager and yield a complete table at the
-        end."""
+        """Wrap the execution with the live manager and yield a table at the end."""
         self.live_manager.start()
-        yield
-        self.live_manager.stop(transient=True)
-        table = self._generate_table(
-            reduce_table=False, sort_table=self.sort_final_table, add_caption=False
-        )
-        if table is not None:
-            console.print(table)
+        try:
+            return (yield)
+        finally:
+            self.live_manager.stop(transient=True)
+            table = self._generate_table(
+                reduce_table=False, sort_table=self.sort_final_table, add_caption=False
+            )
+            if table is not None:
+                console.print(table)
 
     @hookimpl(tryfirst=True)
-    def pytask_execute_task_log_start(self, task: Task) -> bool:
+    def pytask_execute_task_log_start(self, task: PTask) -> bool:
         """Mark a new task as running."""
-        self.update_running_tasks(task)
+        self.add_task(new_running_task=task, status=self.initial_status)
         return True
 
     @hookimpl
     def pytask_execute_task_log_end(self, report: ExecutionReport) -> bool:
         """Mark a task as being finished and update outcome."""
-        self.update_reports(report)
+        self.update_report(report)
         return True
 
     def _generate_table(
@@ -204,11 +210,11 @@ class LiveExecution:
         """
         n_reports_to_display = self.n_entries_in_table - len(self._running_tasks)
 
-        if self.verbose < 2:
+        if self.verbose < 2:  # noqa: PLR2004
             reports = [
                 report
                 for report in self._reports
-                if report["outcome"]
+                if report.outcome
                 not in (
                     TaskOutcome.SKIP,
                     TaskOutcome.SKIP_UNCHANGED,
@@ -227,46 +233,40 @@ class LiveExecution:
             relevant_reports = []
 
         if sort_table:
-            relevant_reports = sorted(
-                relevant_reports, key=lambda report: report["name"]
-            )
+            relevant_reports.sort(key=lambda report: report.name)
 
+        table: Table | None
         if add_caption:
-            caption_kwargs = {
-                "caption": Text(
+            table = Table(
+                caption=Text(
                     f"Completed: {len(self._reports)}/{self.n_tasks}",
                     style=Style(dim=True, italic=False),
                 ),
-                "caption_justify": "right",
-                "caption_style": None,
-            }
+                caption_justify="right",
+                caption_style=None,
+                box=ROUNDED,
+            )
         else:
-            caption_kwargs = {}
+            table = Table(box=ROUNDED)
 
-        table = Table(**caption_kwargs, box=ROUNDED)
         table.add_column("Task", overflow="fold")
         table.add_column("Outcome")
         for report in relevant_reports:
             table.add_row(
-                format_task_id(
-                    report["task"],
-                    editor_url_scheme=self.editor_url_scheme,
-                    short_name=True,
-                ),
-                Text(report["outcome"].symbol, style=report["outcome"].style),
+                format_task_name(report.task, editor_url_scheme=self.editor_url_scheme),
+                Text(report.outcome.symbol, style=report.outcome.style),
             )
-        for task in self._running_tasks.values():
+        for task_entry in self._running_tasks.values():
             table.add_row(
-                format_task_id(
-                    task, editor_url_scheme=self.editor_url_scheme, short_name=True
+                format_task_name(
+                    task_entry.task, editor_url_scheme=self.editor_url_scheme
                 ),
-                "running",
+                task_entry.status.value,
             )
 
         # If the table is empty, do not display anything.
         if table.rows == []:
-            table = None
-
+            return None
         return table
 
     def _update_table(
@@ -281,37 +281,44 @@ class LiveExecution:
         )
         self.live_manager.update(table)
 
-    def update_running_tasks(self, new_running_task: Task) -> None:
+    def add_task(self, new_running_task: PTask, status: TaskExecutionStatus) -> None:
         """Add a new running task."""
-        self._running_tasks[new_running_task.name] = new_running_task
+        self._running_tasks[new_running_task.signature] = _TaskEntry(
+            task=new_running_task, status=status
+        )
         self._update_table()
 
-    def update_reports(self, new_report: ExecutionReport) -> None:
+    def update_task(self, signature: str, status: TaskExecutionStatus) -> None:
+        """Update the status of a running task."""
+        self._running_tasks[signature].status = status
+        self._update_table()
+
+    def update_report(self, new_report: ExecutionReport) -> None:
         """Update the status of a running task by adding its report."""
-        self._running_tasks.pop(new_report.task.name)
+        self._running_tasks.pop(new_report.task.signature)
         self._reports.append(
-            {
-                "name": new_report.task.short_name,
-                "outcome": new_report.outcome,
-                "task": new_report.task,
-            }
+            _ReportEntry(
+                name=new_report.task.name,
+                outcome=new_report.outcome,
+                task=new_report.task,
+            )
         )
         self._update_table()
 
 
-@attr.s(eq=False, kw_only=True)
+@define(eq=False, kw_only=True)
 class LiveCollection:
     """A class for managing the live status during the collection."""
 
-    live_manager = attr.ib(type=LiveManager)
-    _n_collected_tasks = attr.ib(default=0, type=int)
-    _n_errors = attr.ib(default=0, type=int)
+    live_manager: LiveManager
+    _n_collected_tasks: int = 0
+    _n_errors: int = 0
 
-    @hookimpl(hookwrapper=True)
+    @hookimpl(wrapper=True)
     def pytask_collect(self) -> Generator[None, None, None]:
         """Start the status of the collection."""
         self.live_manager.start()
-        yield
+        return (yield)
 
     @hookimpl
     def pytask_collect_file_log(self, reports: list[CollectionReport]) -> None:
@@ -319,16 +326,14 @@ class LiveCollection:
         self._update_statistics(reports)
         self._update_status()
 
-    @hookimpl(hookwrapper=True)
+    @hookimpl(wrapper=True)
     def pytask_collect_log(self) -> Generator[None, None, None]:
         """Stop the live display when all tasks have been collected."""
         self.live_manager.stop(transient=True)
-        yield
+        return (yield)
 
     def _update_statistics(self, reports: list[CollectionReport]) -> None:
         """Update the statistics on collected tasks and errors."""
-        if reports is None:
-            reports = []
         for report in reports:
             if report.outcome == CollectionOutcome.SUCCESS:
                 self._n_collected_tasks += 1
@@ -345,5 +350,4 @@ class LiveCollection:
         msg = f"Collected {self._n_collected_tasks} tasks."
         if self._n_errors > 0:
             msg += f" {self._n_errors} errors."
-        status = Status(msg, spinner="dots")
-        return status
+        return Status(msg, spinner="dots")
