@@ -1,30 +1,35 @@
 """Add general logging capabilities."""
+
 from __future__ import annotations
 
+import contextlib
 import platform
 import sys
-import warnings
+from typing import TYPE_CHECKING
 from typing import Any
 from typing import NamedTuple
 
-import _pytask
 import click
 import pluggy
-from _pytask.config import hookimpl
-from _pytask.console import console
-from _pytask.console import IS_WINDOWS_TERMINAL
-from _pytask.outcomes import CollectionOutcome
-from _pytask.outcomes import TaskOutcome
-from _pytask.session import Session
-from _pytask.shared import convert_truthy_or_falsy_to_bool
-from _pytask.shared import get_first_non_none_value
 from rich.text import Text
 
+import _pytask
+from _pytask.capture_utils import ShowCapture
+from _pytask.console import console
+from _pytask.pluginmanager import hookimpl
+from _pytask.reports import ExecutionReport
+from _pytask.traceback import Traceback
 
-try:
+if TYPE_CHECKING:
     from pluggy._manager import DistFacade
-except ImportError:
-    from pluggy.manager import DistFacade
+
+    from _pytask.outcomes import CollectionOutcome
+    from _pytask.outcomes import TaskOutcome
+    from _pytask.session import Session
+
+
+with contextlib.suppress(ImportError):
+    pass
 
 
 class _TimeUnit(NamedTuple):
@@ -39,56 +44,30 @@ def pytask_extend_command_line_interface(cli: click.Group) -> None:
     show_locals_option = click.Option(
         ["--show-locals"],
         is_flag=True,
-        default=None,
+        default=False,
         help="Show local variables in tracebacks.",
     )
     cli.commands["build"].params.append(show_locals_option)
 
 
 @hookimpl
-def pytask_parse_config(
-    config: dict[str, Any],
-    config_from_file: dict[str, Any],
-    config_from_cli: dict[str, Any],
-) -> None:
-    """Parse configuration."""
-    config["show_locals"] = get_first_non_none_value(
-        config_from_cli,
-        config_from_file,
-        key="show_locals",
-        default=False,
-        callback=convert_truthy_or_falsy_to_bool,
-    )
-    config["editor_url_scheme"] = get_first_non_none_value(
-        config_from_cli,
-        config_from_file,
-        key="editor_url_scheme",
-        default="file",
-        callback=lambda x: None if x in (None, "none", "None") else str(x),
-    )
-    if config["editor_url_scheme"] not in ("no_link", "file") and IS_WINDOWS_TERMINAL:
-        config["editor_url_scheme"] = "file"
-        warnings.warn(
-            "Windows Terminal does not support url schemes to applications, yet."
-            "See https://github.com/pytask-dev/pytask/issues/171 for more information. "
-            "Resort to `editor_url_scheme='file'`."
-        )
-    config["show_traceback"] = get_first_non_none_value(
-        config_from_cli,
-        config_from_file,
-        key="show_traceback",
-        default=True,
-        callback=convert_truthy_or_falsy_to_bool,
-    )
+def pytask_post_parse(config: dict[str, Any]) -> None:
+    # Set class variables on traceback object.
+    Traceback._show_locals = config["show_locals"]
+    # Set class variables on Executionreport.
+    ExecutionReport.editor_url_scheme = config["editor_url_scheme"]
+    ExecutionReport.show_capture = config["show_capture"]
+    ExecutionReport.show_locals = config["show_locals"]
 
 
 @hookimpl
 def pytask_log_session_header(session: Session) -> None:
     """Log the header of a pytask session."""
-    console.rule("Start pytask session", style=None)
+    console.rule("Start pytask session", style="default")
     console.print(
         f"Platform: {sys.platform} -- Python {platform.python_version()}, "
-        f"pytask {_pytask.__version__}, pluggy {pluggy.__version__}"
+        f"pytask {_pytask.__version__}, pluggy {pluggy.__version__}",
+        highlight=False,
     )
     console.print(f"Root: {session.config['root']}")
     if session.config["config"] is not None:
@@ -103,7 +82,7 @@ def pytask_log_session_header(session: Session) -> None:
 
 
 def _format_plugin_names_and_versions(
-    plugininfo: list[tuple[str, DistFacade]]
+    plugininfo: list[tuple[str, DistFacade]],
 ) -> list[str]:
     """Format name and version of loaded plugins."""
     values: list[str] = []
@@ -111,8 +90,7 @@ def _format_plugin_names_and_versions(
         # Gets us name and version!
         name = f"{dist.project_name}-{dist.version}"
         # Questionable convenience, but it keeps things short.
-        if name.startswith("pytask-"):
-            name = name[7:]
+        name = name.removeprefix("pytask-")
         # We decided to print python package names they can have more than one plugin.
         if name not in values:
             values.append(name)
@@ -132,6 +110,15 @@ def pytask_log_session_footer(
     console.rule(message, style=outcome.style)
 
 
+@hookimpl
+def pytask_unconfigure() -> None:
+    """Reset class variables."""
+    Traceback._show_locals = False
+    ExecutionReport.editor_url_scheme = "file"
+    ExecutionReport.show_capture = ShowCapture.ALL
+    ExecutionReport.show_locals = False
+
+
 _TIME_UNITS: list[_TimeUnit] = [
     _TimeUnit(singular="day", plural="days", short="d", in_seconds=86400),
     _TimeUnit(singular="hour", plural="hours", short="h", in_seconds=3600),
@@ -150,12 +137,11 @@ def _format_duration(duration: float) -> str:
             i for i in duration_tuples if i[1] not in ("second", "seconds")
         ]
 
-    formatted_duration = ", ".join([" ".join(map(str, i)) for i in duration_tuples])
-    return formatted_duration
+    return ", ".join([" ".join(map(str, i)) for i in duration_tuples])
 
 
-def _humanize_time(
-    amount: int | float, unit: str, short_label: bool = False
+def _humanize_time(  # noqa: C901, PLR0912
+    amount: float, unit: str, short_label: bool = False
 ) -> list[tuple[float, str]]:
     """Humanize the time.
 
@@ -181,7 +167,8 @@ def _humanize_time(
             index = i
             break
     else:
-        raise ValueError(f"The time unit {unit!r} is not known.")
+        msg = f"The time unit {unit!r} is not known."
+        raise ValueError(msg)
 
     seconds = amount * _TIME_UNITS[index].in_seconds
     result: list[tuple[float, str]] = []
@@ -200,17 +187,16 @@ def _humanize_time(
                 label = time_unit.plural
             result.append((last_seconds, label))
 
-        elif whole_units >= 1:
-            if time_unit.singular != "seconds":
-                if short_label:
-                    label = time_unit.short
-                elif whole_units == 1:
-                    label = time_unit.singular
-                else:
-                    label = time_unit.plural
+        elif whole_units >= 1 and time_unit.singular != "seconds":
+            if short_label:
+                label = time_unit.short
+            elif whole_units == 1:
+                label = time_unit.singular
+            else:
+                label = time_unit.plural
 
-                result.append((whole_units, label))
-                remaining_seconds -= whole_units * time_unit.in_seconds
+            result.append((whole_units, label))
+            remaining_seconds -= whole_units * time_unit.in_seconds
 
     if not result:
         result.append(
