@@ -1,68 +1,67 @@
+"""Contains the main code for the markers plugin."""
+
 from __future__ import annotations
 
 import sys
-from typing import AbstractSet
-from typing import Any
-from typing import Set
 from typing import TYPE_CHECKING
+from typing import Any
 
-import attr
 import click
-import networkx as nx
+from attrs import define
+from rich.table import Table
+
 from _pytask.click import ColoredCommand
-from _pytask.config import hookimpl
 from _pytask.console import console
-from _pytask.dag import task_and_preceding_tasks
+from _pytask.dag_utils import task_and_preceding_tasks
 from _pytask.exceptions import ConfigurationError
 from _pytask.mark.expression import Expression
 from _pytask.mark.expression import ParseError
-from _pytask.mark.structures import Mark
 from _pytask.mark.structures import MARK_GEN
+from _pytask.mark.structures import Mark
 from _pytask.mark.structures import MarkDecorator
 from _pytask.mark.structures import MarkGenerator
-from _pytask.nodes import Task
 from _pytask.outcomes import ExitCode
-from _pytask.pluginmanager import get_plugin_manager
+from _pytask.pluginmanager import hookimpl
+from _pytask.pluginmanager import storage
 from _pytask.session import Session
-from _pytask.shared import convert_truthy_or_falsy_to_bool
-from _pytask.shared import get_first_non_none_value
-from rich.table import Table
-
+from _pytask.shared import parse_markers
 
 if TYPE_CHECKING:
+    from collections.abc import Set as AbstractSet
     from typing import NoReturn
 
+    import networkx as nx
+
+    from _pytask.node_protocols import PTask
+
 
 __all__ = [
+    "MARK_GEN",
     "Expression",
     "Mark",
     "MarkDecorator",
     "MarkGenerator",
-    "MARK_GEN",
     "ParseError",
+    "select_by_after_keyword",
+    "select_by_keyword",
+    "select_by_mark",
+    "select_tasks_by_marks_and_expressions",
 ]
 
 
 @click.command(cls=ColoredCommand)
-def markers(**config_from_cli: Any) -> NoReturn:
+def markers(**raw_config: Any) -> NoReturn:
     """Show all registered markers."""
-    config_from_cli["command"] = "markers"
+    raw_config["command"] = "markers"
+    pm = storage.get()
 
     try:
-        # Duplication of the same mechanism in :func:`pytask.main.main`.
-        pm = get_plugin_manager()
-        from _pytask import cli
-
-        pm.register(cli)
-        pm.hook.pytask_add_hooks(pm=pm)
-
-        config = pm.hook.pytask_configure(pm=pm, config_from_cli=config_from_cli)
+        config = pm.hook.pytask_configure(pm=pm, raw_config=raw_config)
         session = Session.from_config(config)
 
-    except (ConfigurationError, Exception):
+    except (ConfigurationError, Exception):  # noqa: BLE001  # pragma: no cover
         console.print_exception()
-        session = Session({}, None)
-        session.exit_code = ExitCode.CONFIGURATION_FAILED
+        session = Session(exit_code=ExitCode.CONFIGURATION_FAILED)
 
     else:
         table = Table("Marker", "Description", leading=1)
@@ -72,18 +71,21 @@ def markers(**config_from_cli: Any) -> NoReturn:
 
         console.print(table)
 
+    session.hook.pytask_unconfigure(session=session)
     sys.exit(session.exit_code)
 
 
 @hookimpl
 def pytask_extend_command_line_interface(cli: click.Group) -> None:
     """Add marker related options."""
+    cli.add_command(markers)
+
     additional_build_parameters = [
         click.Option(
             ["--strict-markers"],
             is_flag=True,
             help="Raise errors for unknown markers.",
-            default=None,
+            default=False,
         ),
         click.Option(
             ["-m", "marker_expression"],
@@ -98,68 +100,22 @@ def pytask_extend_command_line_interface(cli: click.Group) -> None:
             help="Select tasks via expressions on task ids.",
         ),
     ]
-    cli.commands["build"].params.extend(additional_build_parameters)
-    cli.commands["clean"].params.extend(additional_build_parameters)
-    cli.commands["collect"].params.extend(additional_build_parameters)
-
-    cli.add_command(markers)
+    for command in ("build", "clean", "collect"):
+        cli.commands[command].params.extend(additional_build_parameters)
 
 
 @hookimpl
-def pytask_parse_config(
-    config: dict[str, Any],
-    config_from_cli: dict[str, Any],
-    config_from_file: dict[str, Any],
-) -> None:
+def pytask_parse_config(config: dict[str, Any]) -> None:
     """Parse marker related options."""
-    markers = _read_marker_mapping(config_from_file.get("markers", {}))
-    config["markers"] = {**markers, **config["markers"]}
-    config["strict_markers"] = get_first_non_none_value(
-        config,
-        config_from_file,
-        config_from_cli,
-        key="strict_markers",
-        default=False,
-        callback=convert_truthy_or_falsy_to_bool,
-    )
-
-    config["expression"] = config_from_cli.get("expression")
-    config["marker_expression"] = config_from_cli.get("marker_expression")
-
     MARK_GEN.config = config
 
 
-def _read_marker_mapping(x: dict[str, str] | str) -> dict[str, str]:
-    """Read marker descriptions from configuration file."""
-    if isinstance(x, dict):
-        mapping = {k.strip(): v.strip() for k, v in x.items()}
-    elif isinstance(x, list):
-        mapping = {k.strip(): "" for k in x}
-    elif isinstance(x, str):
-        # Split by newlines and remove empty strings.
-        lines = filter(bool, x.split("\n"))
-        mapping = {}
-        for line in lines:
-            try:
-                key, value = line.split(":")
-            except ValueError:
-                key = line
-                value = ""
-
-            mapping[key.strip()] = value.strip()
-    else:
-        raise TypeError(f"Unknown type {type(x)!r}. Expected dict or str.")
-
-    for key in mapping:
-        if not key.isidentifier():
-            raise ValueError(
-                f"{key} is not a valid Python name and cannot be used as a marker."
-            )
-
-    return mapping
+@hookimpl
+def pytask_post_parse(config: dict[str, Any]) -> None:
+    config["markers"] = parse_markers(config["markers"])
 
 
-@attr.s(slots=True)
+@define(slots=True)
 class KeywordMatcher:
     """A matcher for keywords.
 
@@ -174,16 +130,14 @@ class KeywordMatcher:
 
     """
 
-    _names = attr.ib(type=AbstractSet[str])
+    _names: AbstractSet[str]
 
     @classmethod
-    def from_task(cls, task: Task) -> KeywordMatcher:
+    def from_task(cls, task: PTask) -> KeywordMatcher:
         mapped_names = {task.name}
 
         # Add the names attached to the current function through direct assignment.
-        function_obj = task.function
-        if function_obj:
-            mapped_names.update(function_obj.__dict__)
+        mapped_names.update(task.function.__dict__)
 
         # Add the markers to the keywords as we no longer handle them correctly.
         mapped_names.update(mark.name for mark in task.markers)
@@ -194,13 +148,10 @@ class KeywordMatcher:
         subname = subname.lower()
         names = (name.lower() for name in self._names)
 
-        for name in names:
-            if subname in name:
-                return True
-        return False
+        return any(subname in name for name in names)
 
 
-def select_by_keyword(session: Session, dag: nx.DiGraph) -> set[str]:
+def select_by_keyword(session: Session, dag: nx.DiGraph) -> set[str] | None:
     """Deselect tests by keywords."""
     keywordexpr = session.config["expression"]
     if not keywordexpr:
@@ -209,19 +160,34 @@ def select_by_keyword(session: Session, dag: nx.DiGraph) -> set[str]:
     try:
         expression = Expression.compile_(keywordexpr)
     except ParseError as e:
-        raise ValueError(
-            f"Wrong expression passed to '-k': {keywordexpr}: {e}"
-        ) from None
+        msg = f"Wrong expression passed to '-k': {keywordexpr}: {e}"
+        raise ValueError(msg) from None
 
     remaining: set[str] = set()
     for task in session.tasks:
         if keywordexpr and expression.evaluate(KeywordMatcher.from_task(task)):
-            remaining.update(task_and_preceding_tasks(task.name, dag))
+            remaining.update(task_and_preceding_tasks(task.signature, dag))
 
     return remaining
 
 
-@attr.s(slots=True)
+def select_by_after_keyword(session: Session, after: str) -> set[str]:
+    """Select tasks defined by the after keyword."""
+    try:
+        expression = Expression.compile_(after)
+    except ParseError as e:
+        msg = f"Wrong expression passed to 'after': {after}: {e}"
+        raise ValueError(msg) from None
+
+    ancestors: set[str] = set()
+    for task in session.tasks:
+        if after and expression.evaluate(KeywordMatcher.from_task(task)):
+            ancestors.add(task.signature)
+
+    return ancestors
+
+
+@define(slots=True)
 class MarkMatcher:
     """A matcher for markers which are present.
 
@@ -229,10 +195,10 @@ class MarkMatcher:
 
     """
 
-    own_mark_names = attr.ib(type=Set[str])
+    own_mark_names: set[str]
 
     @classmethod
-    def from_task(cls, task: Task) -> MarkMatcher:
+    def from_task(cls, task: PTask) -> MarkMatcher:
         mark_names = {mark.name for mark in task.markers}
         return cls(mark_names)
 
@@ -240,7 +206,7 @@ class MarkMatcher:
         return name in self.own_mark_names
 
 
-def select_by_mark(session: Session, dag: nx.DiGraph) -> set[str]:
+def select_by_mark(session: Session, dag: nx.DiGraph) -> set[str] | None:
     """Deselect tests by marks."""
     matchexpr = session.config["marker_expression"]
     if not matchexpr:
@@ -249,12 +215,13 @@ def select_by_mark(session: Session, dag: nx.DiGraph) -> set[str]:
     try:
         expression = Expression.compile_(matchexpr)
     except ParseError as e:
-        raise ValueError(f"Wrong expression passed to '-m': {matchexpr}: {e}") from None
+        msg = f"Wrong expression passed to '-m': {matchexpr}: {e}"
+        raise ValueError(msg) from None
 
     remaining: set[str] = set()
     for task in session.tasks:
         if expression.evaluate(MarkMatcher.from_task(task)):
-            remaining.update(task_and_preceding_tasks(task.name, dag))
+            remaining.update(task_and_preceding_tasks(task.signature, dag))
 
     return remaining
 
@@ -262,13 +229,13 @@ def select_by_mark(session: Session, dag: nx.DiGraph) -> set[str]:
 def _deselect_others_with_mark(
     session: Session, remaining: set[str], mark: Mark
 ) -> None:
+    """Deselect tasks."""
     for task in session.tasks:
-        if task.name not in remaining:
+        if task.signature not in remaining:
             task.markers.append(mark)
 
 
-@hookimpl
-def pytask_resolve_dependencies_modify_dag(session: Session, dag: nx.DiGraph) -> None:
+def select_tasks_by_marks_and_expressions(session: Session, dag: nx.DiGraph) -> None:
     """Modify the tasks which are executed with expressions and markers."""
     remaining = select_by_keyword(session, dag)
     if remaining is not None:
