@@ -1,29 +1,41 @@
 from __future__ import annotations
 
 import warnings
+from typing import TYPE_CHECKING
 from typing import Any
-from typing import Callable
-from typing import Iterable
-from typing import Mapping
 
-import attr
+from attrs import define
+from attrs import field
+from attrs import validators
+
+from _pytask.mark_utils import get_all_marks
 from _pytask.models import CollectionMetadata
+from _pytask.typing import is_task_function
+
+if TYPE_CHECKING:
+    from collections.abc import Callable
+    from collections.abc import Iterable
+    from collections.abc import Mapping
 
 
-def is_task_function(func: Any) -> bool:
-    return callable(func) and getattr(func, "__name__", "<lambda>") != "<lambda>"
-
-
-@attr.s(frozen=True, auto_attribs=True)
+@define(frozen=True)
 class Mark:
-    """A class for a mark containing the name, positional and keyword arguments."""
+    """A class for a mark containing the name, positional and keyword arguments.
+
+    Attributes
+    ----------
+    name
+        Name of the mark.
+    args
+        Positional arguments of the mark decorator.
+    kwargs
+        Keyword arguments of the mark decorator.
+
+    """
 
     name: str
-    """str: Name of the mark."""
     args: tuple[Any, ...]
-    """Tuple[Any]: Positional arguments of the mark decorator."""
     kwargs: Mapping[str, Any]
-    """Mapping[str, Any]: Keyword arguments of the mark decorator."""
 
     def combined_with(self, other: Mark) -> Mark:
         """Return a new Mark which is a combination of this Mark and another Mark.
@@ -32,7 +44,7 @@ class Mark:
 
         Parameters
         ----------
-        other : pytask.mark.structures.Mark
+        other
             The mark to combine with.
 
         Returns
@@ -45,7 +57,7 @@ class Mark:
         return Mark(self.name, self.args + other.args, {**self.kwargs, **other.kwargs})
 
 
-@attr.s
+@define
 class MarkDecorator:
     """A decorator for applying a mark on task function.
 
@@ -81,7 +93,7 @@ class MarkDecorator:
 
     """
 
-    mark = attr.ib(type=Mark, validator=attr.validators.instance_of(Mark))
+    mark: Mark = field(validator=validators.instance_of(Mark))
 
     @property
     def name(self) -> str:
@@ -123,10 +135,7 @@ class MarkDecorator:
 
 def get_unpacked_marks(obj: Callable[..., Any]) -> list[Mark]:
     """Obtain the unpacked marks that are stored on an object."""
-    if hasattr(obj, "pytask_meta"):
-        mark_list = obj.pytask_meta.markers
-    else:
-        mark_list = []
+    mark_list = get_all_marks(obj)
     return normalize_mark_list(mark_list)
 
 
@@ -144,8 +153,9 @@ def normalize_mark_list(mark_list: Iterable[Mark | MarkDecorator]) -> list[Mark]
     """
     extracted = [getattr(mark, "mark", mark) for mark in mark_list]
     for mark in extracted:
-        if not isinstance(mark, Mark):
-            raise TypeError(f"Got {mark!r} instead of Mark.")
+        if not isinstance(mark, Mark):  # pragma: no cover
+            msg = f"Got {mark!r} instead of Mark."
+            raise TypeError(msg)
     return [x for x in extracted if isinstance(x, Mark)]
 
 
@@ -157,16 +167,23 @@ def store_mark(obj: Callable[..., Any], mark: Mark) -> None:
     """
     assert isinstance(mark, Mark), mark
     if hasattr(obj, "pytask_meta"):
-        obj.pytask_meta.markers = get_unpacked_marks(obj) + [mark]
+        obj.pytask_meta.markers = [*get_unpacked_marks(obj), mark]
     else:
         obj.pytask_meta = CollectionMetadata(  # type: ignore[attr-defined]
             markers=[mark]
         )
 
 
+_DEPRECATION_DECORATOR = """'@pytask.mark.{}' was removed in pytask v0.5.0. To upgrade \
+your project to the new syntax, read the tutorial on product and dependencies: \
+https://tinyurl.com/pytask-deps-prods.
+"""
+
+
 class MarkGenerator:
-    """Factory for :class:`MarkDecorator` objects - exposed as a :class:`pytask.mark`
-    singleton instance.
+    """Factory for :class:`MarkDecorator` objects.
+
+    Exposed as a :class:`pytask.mark` singleton instance.
 
     Example
     -------
@@ -185,28 +202,39 @@ class MarkGenerator:
 
     def __getattr__(self, name: str) -> MarkDecorator | Any:
         if name[0] == "_":
-            raise AttributeError("Marker name must NOT start with underscore")
+            msg = "Marker name must NOT start with underscore"
+            raise AttributeError(msg)
 
-        if self.config is not None:
-            # If the name is not in the set of known marks after updating,
-            # then it really is time to issue a warning or an error.
-            if name not in self.config["markers"]:
-                if self.config["strict_markers"]:
-                    raise ValueError(f"Unknown pytask.mark.{name}.")
-                # Raise a specific error for common misspellings of "parametrize".
-                if name in ("parameterize", "parametrise", "parameterise"):
-                    warnings.warn(f"Unknown {name!r} mark, did you mean 'parametrize'?")
-
-                warnings.warn(
-                    f"Unknown pytask.mark.{name} - is this a typo? You can register "
-                    "custom marks to avoid this warning.",
-                    stacklevel=2,
-                )
+        if name in ("depends_on", "produces"):
+            raise RuntimeError(_DEPRECATION_DECORATOR.format(name))
 
         if name == "task":
-            from _pytask.task_utils import task
+            msg = (
+                "'@pytask.mark.task' is removed. Use '@task' with 'from pytask import "
+                "task' instead."
+            )
+            raise RuntimeError(msg)
 
-            return task
+        # If the name is not in the set of known marks after updating,
+        # then it really is time to issue a warning or an error.
+        if self.config is not None and name not in self.config["markers"]:
+            if name in ("parametrize", "parameterize", "parametrise", "parameterise"):
+                msg = (
+                    "@pytask.mark.parametrize has been removed since pytask v0.4. "
+                    "Upgrade your parametrized tasks to the new syntax defined in "
+                    "https://tinyurl.com/pytask-loops or revert to v0.3."
+                )
+                raise NotImplementedError(msg) from None
+
+            if self.config["strict_markers"]:
+                msg = f"Unknown pytask.mark.{name}."
+                raise ValueError(msg)
+
+            warnings.warn(
+                f"Unknown pytask.mark.{name} - is this a typo? You can register "
+                "custom marks to avoid this warning.",
+                stacklevel=2,
+            )
 
         return MarkDecorator(Mark(name, (), {}))
 
