@@ -1,29 +1,30 @@
-"""This module contains code related to outcomes."""
+"""Contains code related to outcomes."""
+
 from __future__ import annotations
 
-from enum import auto
 from enum import Enum
 from enum import IntEnum
-from typing import Sequence
+from enum import auto
 from typing import TYPE_CHECKING
 
-
 if TYPE_CHECKING:
-    from _pytask.report import CollectionReport
-    from _pytask.report import ExecutionReport
+    from collections.abc import Sequence
+
+    from _pytask.reports import CollectionReport
+    from _pytask.reports import ExecutionReport
 
 
 __all__ = [
-    "count_outcomes",
     "CollectionOutcome",
     "Exit",
     "ExitCode",
     "Persisted",
     "PytaskOutcome",
     "Skipped",
-    "SkippedUnchanged",
     "SkippedAncestorFailed",
+    "SkippedUnchanged",
     "TaskOutcome",
+    "count_outcomes",
 ]
 
 
@@ -94,6 +95,8 @@ class TaskOutcome(Enum):
         source files and products have not changed.
     SUCCESS
         Outcome for task which was executed successfully.
+    WOULD_BE_EXECUTED
+        Outcome for tasks which would be executed.
 
     """
 
@@ -168,13 +171,13 @@ class TaskOutcome(Enum):
 
 def count_outcomes(
     reports: Sequence[CollectionReport | ExecutionReport],
-    outcome_enum: type[CollectionOutcome] | type[TaskOutcome],
+    outcome_enum: type[CollectionOutcome | TaskOutcome],
 ) -> dict[Enum, int]:
     """Count how often an outcome occurred.
 
     Examples
     --------
-    >>> from _pytask.outcomes import CollectionOutcome, TaskOutcome
+    >>> from _pytask.outcomes import CollectionOutcome, TaskOutcome, count_outcomes
     >>> count_outcomes([], CollectionOutcome)
     {<CollectionOutcome.SUCCESS: 1>: 0, <CollectionOutcome.FAIL: 2>: 0}
 
@@ -199,8 +202,8 @@ class ExitCode(IntEnum):
     COLLECTION_FAILED = 3
     """Failed while collecting tasks."""
 
-    RESOLVING_DEPENDENCIES_FAILED = 4
-    """Failed while resolving dependencies."""
+    DAG_FAILED = 4
+    """Failed while building the DAG."""
 
 
 class PytaskOutcome(Exception):
