@@ -1,56 +1,61 @@
-"""This module contains the code to profile the execution."""
+"""Contains the code to profile the execution."""
+
 from __future__ import annotations
 
 import csv
+import enum
 import json
 import sys
 import time
 from contextlib import suppress
-from enum import Enum
-from pathlib import Path
-from types import TracebackType
-from typing import Any
-from typing import Generator
 from typing import TYPE_CHECKING
+from typing import Any
 
 import click
+from rich.table import Table
+from sqlalchemy.orm import Mapped
+from sqlalchemy.orm import mapped_column
+
 from _pytask.click import ColoredCommand
-from _pytask.config import hookimpl
-from _pytask.config_utils import parse_click_choice
+from _pytask.click import EnumChoice
 from _pytask.console import console
-from _pytask.console import format_task_id
-from _pytask.database import db
+from _pytask.console import format_task_name
+from _pytask.dag import create_dag
+from _pytask.database_utils import BaseTable
+from _pytask.database_utils import DatabaseSession
 from _pytask.exceptions import CollectionError
 from _pytask.exceptions import ConfigurationError
-from _pytask.nodes import FilePathNode
-from _pytask.nodes import Task
+from _pytask.node_protocols import PPathNode
+from _pytask.node_protocols import PTask
 from _pytask.outcomes import ExitCode
 from _pytask.outcomes import TaskOutcome
-from _pytask.pluginmanager import get_plugin_manager
-from _pytask.report import ExecutionReport
+from _pytask.pluginmanager import hookimpl
+from _pytask.pluginmanager import storage
 from _pytask.session import Session
-from _pytask.shared import get_first_non_none_value
-from _pytask.traceback import render_exc_info
-from pony import orm
-from rich.table import Table
-
+from _pytask.traceback import Traceback
 
 if TYPE_CHECKING:
+    from collections.abc import Generator
+    from pathlib import Path
     from typing import NoReturn
 
+    from _pytask.reports import ExecutionReport
 
-class _ExportFormats(str, Enum):
+
+class _ExportFormats(enum.Enum):
     NO = "no"
     JSON = "json"
     CSV = "csv"
 
 
-class Runtime(db.Entity):  # type: ignore
+class Runtime(BaseTable):
     """Record of runtimes of tasks."""
 
-    task = orm.PrimaryKey(str)
-    date = orm.Required(float)
-    duration = orm.Required(float)
+    __tablename__ = "runtime"
+
+    task: Mapped[str] = mapped_column(primary_key=True)
+    date: Mapped[float]
+    duration: Mapped[float]
 
 
 @hookimpl(tryfirst=True)
@@ -59,19 +64,6 @@ def pytask_extend_command_line_interface(cli: click.Group) -> None:
     cli.add_command(profile)
 
 
-@hookimpl
-def pytask_parse_config(
-    config: dict[str, Any], config_from_cli: dict[str, Any]
-) -> None:
-    """Parse the configuration."""
-    config["export"] = get_first_non_none_value(
-        config_from_cli,
-        key="export",
-        default=_ExportFormats.NO,
-        callback=parse_click_choice("export", _ExportFormats),
-    )
-
-
 @hookimpl
 def pytask_post_parse(config: dict[str, Any]) -> None:
     """Register the export option."""
@@ -80,13 +72,14 @@ def pytask_post_parse(config: dict[str, Any]) -> None:
     config["pm"].register(FileSizeNameSpace)
 
 
-@hookimpl(hookwrapper=True)
-def pytask_execute_task(task: Task) -> Generator[None, None, None]:
+@hookimpl(wrapper=True)
+def pytask_execute_task(task: PTask) -> Generator[None, None, None]:
     """Attach the duration of the execution to the task."""
     start = time.time()
-    yield
+    result = yield
     end = time.time()
     task.attributes["duration"] = (start, end)
+    return result
 
 
 @hookimpl
@@ -95,56 +88,48 @@ def pytask_execute_task_process_report(report: ExecutionReport) -> None:
     task = report.task
     duration = task.attributes.get("duration")
     if report.outcome == TaskOutcome.SUCCESS and duration is not None:
-        _create_or_update_runtime(task.name, *duration)
+        _create_or_update_runtime(task.signature, *duration)
 
 
-@orm.db_session
-def _create_or_update_runtime(task_name: str, start: float, end: float) -> None:
+def _create_or_update_runtime(task_signature: str, start: float, end: float) -> None:
     """Create or update a runtime entry."""
-    try:
-        runtime = Runtime[task_name]  # type: ignore
-    except orm.ObjectNotFound:
-        Runtime(task=task_name, date=start, duration=end - start)
-    else:
-        for attr, val in (("date", start), ("duration", end - start)):
-            setattr(runtime, attr, val)
+    with DatabaseSession() as session:
+        runtime = session.get(Runtime, task_signature)
+
+        if not runtime:
+            session.add(Runtime(task=task_signature, date=start, duration=end - start))
+        else:
+            for attr, val in (("date", start), ("duration", end - start)):
+                setattr(runtime, attr, val)
+
+        session.commit()
 
 
 @click.command(cls=ColoredCommand)
 @click.option(
     "--export",
-    type=click.Choice(_ExportFormats),  # type: ignore[arg-type]
-    default=None,
-    help="Export the profile in the specified format. [dim]\\[default: no][/]",
+    type=EnumChoice(_ExportFormats),
+    default=_ExportFormats.NO,
+    help="Export the profile in the specified format.",
 )
-def profile(**config_from_cli: Any) -> NoReturn:
-    """Show information about tasks like runtime and memory consumption of products."""
-    config_from_cli["command"] = "profile"
+def profile(**raw_config: Any) -> NoReturn:
+    """Show information about resource consumption."""
+    pm = storage.get()
+    raw_config["command"] = "profile"
 
     try:
-        # Duplication of the same mechanism in :func:`pytask.main.main`.
-        pm = get_plugin_manager()
-        from _pytask import cli
-
-        pm.register(cli)
-        pm.hook.pytask_add_hooks(pm=pm)
-
-        config = pm.hook.pytask_configure(pm=pm, config_from_cli=config_from_cli)
+        config = pm.hook.pytask_configure(pm=pm, raw_config=raw_config)
         session = Session.from_config(config)
 
-    except (ConfigurationError, Exception):  # pragma: no cover
-        session = Session({}, None)
-        session.exit_code = ExitCode.CONFIGURATION_FAILED
-        exc_info: tuple[
-            type[BaseException], BaseException, TracebackType | None
-        ] = sys.exc_info()
-        console.print(render_exc_info(*exc_info, show_locals=config["show_locals"]))
+    except (ConfigurationError, Exception):  # noqa: BLE001  # pragma: no cover
+        session = Session(exit_code=ExitCode.CONFIGURATION_FAILED)
+        console.print(Traceback(sys.exc_info()))
 
     else:
         try:
             session.hook.pytask_log_session_header(session=session)
             session.hook.pytask_collect(session=session)
-            session.hook.pytask_resolve_dependencies(session=session)
+            session.dag = create_dag(session=session)
 
             profile: dict[str, dict[str, Any]] = {
                 task.name: {} for task in session.tasks
@@ -163,16 +148,17 @@ def profile(**config_from_cli: Any) -> NoReturn:
         except CollectionError:  # pragma: no cover
             session.exit_code = ExitCode.COLLECTION_FAILED
 
-        except Exception:  # pragma: no cover
+        except Exception:  # noqa: BLE001 pragma: no cover
             session.exit_code = ExitCode.FAILED
             console.print_exception()
             console.rule(style="failed")
 
+    session.hook.pytask_unconfigure(session=session)
     sys.exit(session.exit_code)
 
 
 def _print_profile_table(
-    profile: dict[str, dict[str, Any]], tasks: list[Task], config: dict[str, Any]
+    profile: dict[str, dict[str, Any]], tasks: list[PTask], config: dict[str, Any]
 ) -> None:
     """Print the profile table."""
     name_to_task = {task.name: task for task in tasks}
@@ -185,10 +171,9 @@ def _print_profile_table(
             table.add_column(name, justify="right")
 
         for task_name, info in profile.items():
-            task_id = format_task_id(
+            task_id = format_task_name(
                 task=name_to_task[task_name],
                 editor_url_scheme=config["editor_url_scheme"],
-                short_name=True,
             )
             infos = [str(i) for i in info.values()]
             table.add_row(task_id, *infos)
@@ -204,20 +189,21 @@ class DurationNameSpace:
     @staticmethod
     @hookimpl
     def pytask_profile_add_info_on_task(
-        tasks: list[Task], profile: dict[str, dict[str, Any]]
+        tasks: list[PTask], profile: dict[str, dict[str, Any]]
     ) -> None:
         """Add the runtime for tasks to the profile."""
-        runtimes = _collect_runtimes([task.name for task in tasks])
+        runtimes = _collect_runtimes(tasks)
         for name, duration in runtimes.items():
             profile[name]["Duration (in s)"] = round(duration, 2)
 
 
-@orm.db_session
-def _collect_runtimes(task_names: list[str]) -> dict[str, float]:
+def _collect_runtimes(tasks: list[PTask]) -> dict[str, float]:
     """Collect runtimes."""
-    runtimes = [Runtime.get(task=task_name) for task_name in task_names]
-    runtimes = [r for r in runtimes if r is not None]
-    return {r.task: r.duration for r in runtimes}
+    with DatabaseSession() as session:
+        runtimes = [session.get(Runtime, task.signature) for task in tasks]
+    return {
+        task.name: r.duration for task, r in zip(tasks, runtimes, strict=False) if r
+    }
 
 
 class FileSizeNameSpace:
@@ -226,16 +212,16 @@ class FileSizeNameSpace:
     @staticmethod
     @hookimpl
     def pytask_profile_add_info_on_task(
-        session: Session, tasks: list[Task], profile: dict[str, dict[str, Any]]
+        session: Session, tasks: list[PTask], profile: dict[str, dict[str, Any]]
     ) -> None:
         """Add the total file size of all products for a task."""
         for task in tasks:
-            successors = list(session.dag.successors(task.name))
+            successors = list(session.dag.successors(task.signature))
             if successors:
                 sum_bytes = 0
                 for successor in successors:
                     node = session.dag.nodes[successor]["node"]
-                    if isinstance(node, FilePathNode):
+                    if isinstance(node, PPathNode):
                         with suppress(FileNotFoundError):
                             sum_bytes += node.path.stat().st_size
 
@@ -249,7 +235,7 @@ def _to_human_readable_size(bytes_: int, units: list[str] | None = None) -> str:
     units = [" bytes", " KB", " MB", " GB", " TB"] if units is None else units
     return (
         str(bytes_) + units[0]
-        if bytes_ < 1024 or len(units) == 1
+        if bytes_ < 1024 or len(units) == 1  # noqa: PLR2004
         else _to_human_readable_size(bytes_ >> 10, units[1:])
     )
 
@@ -282,19 +268,20 @@ class ExportNameSpace:
         export = session.config["export"]
 
         if export == _ExportFormats.CSV:
-            _export_to_csv(profile)
+            _export_to_csv(profile, session.config["root"])
         elif export == _ExportFormats.JSON:
-            _export_to_json(profile)
+            _export_to_json(profile, session.config["root"])
         elif export == _ExportFormats.NO:
             pass
         else:  # pragma: no cover
-            raise ValueError(f"The export option {export.value!r} cannot be handled.")
+            msg = f"The export option {export.value!r} cannot be handled."
+            raise ValueError(msg)
 
 
-def _export_to_csv(profile: dict[str, dict[str, Any]]) -> None:
+def _export_to_csv(profile: dict[str, dict[str, Any]], root: Path) -> None:
     """Export profile to csv."""
     info_names = _get_info_names(profile)
-    path = Path.cwd().joinpath("profile.csv")
+    path = root.joinpath("profile.csv")
 
     with path.open("w", newline="") as file:
         writer = csv.writer(file)
@@ -303,10 +290,10 @@ def _export_to_csv(profile: dict[str, dict[str, Any]]) -> None:
             writer.writerow((task_name, *info.values()))
 
 
-def _export_to_json(profile: dict[str, dict[str, Any]]) -> None:
+def _export_to_json(profile: dict[str, dict[str, Any]], root: Path) -> None:
     """Export profile to json."""
     json_ = json.dumps(profile)
-    path = Path.cwd().joinpath("profile.json")
+    path = root.joinpath("profile.json")
     path.write_text(json_)
 
 
