@@ -1,11 +1,11 @@
-"""This module contains everything related to skipping tasks."""
+"""Contains everything related to skipping tasks."""
+
 from __future__ import annotations
 
-from typing import Any
 from typing import TYPE_CHECKING
+from typing import Any
 
-from _pytask.config import hookimpl
-from _pytask.dag import descending_tasks
+from _pytask.dag_utils import descending_tasks
 from _pytask.mark import Mark
 from _pytask.mark_utils import get_marks
 from _pytask.mark_utils import has_mark
@@ -13,22 +13,22 @@ from _pytask.outcomes import Skipped
 from _pytask.outcomes import SkippedAncestorFailed
 from _pytask.outcomes import SkippedUnchanged
 from _pytask.outcomes import TaskOutcome
-from _pytask.traceback import remove_traceback_from_exc_info
-
+from _pytask.pluginmanager import hookimpl
+from _pytask.provisional_utils import collect_provisional_products
 
 if TYPE_CHECKING:
+    from _pytask.node_protocols import PTask
+    from _pytask.reports import ExecutionReport
     from _pytask.session import Session
-    from _pytask.nodes import Task
-    from _pytask.report import ExecutionReport
 
 
 def skip_ancestor_failed(reason: str = "No reason provided.") -> str:
-    """Function to parse information in ``@pytask.mark.skip_ancestor_failed``."""
+    """Parse information in ``@pytask.mark.skip_ancestor_failed``."""
     return reason
 
 
 def skipif(condition: bool, *, reason: str) -> tuple[bool, str]:
-    """Function to parse information in ``@pytask.mark.skipif``."""
+    """Parse information in ``@pytask.mark.skipif``."""
     return condition, reason
 
 
@@ -47,22 +47,15 @@ def pytask_parse_config(config: dict[str, Any]) -> None:
 
 
 @hookimpl
-def pytask_execute_task_setup(task: Task) -> None:
+def pytask_execute_task_setup(session: Session, task: PTask) -> None:
     """Take a short-cut for skipped tasks during setup with an exception."""
     is_unchanged = has_mark(task, "skip_unchanged") and not has_mark(
         task, "would_be_executed"
     )
-    if is_unchanged:
+    if is_unchanged and not session.config["force"]:
+        collect_provisional_products(session, task)
         raise SkippedUnchanged
 
-    ancestor_failed_marks = get_marks(task, "skip_ancestor_failed")
-    if ancestor_failed_marks:
-        message = "\n".join(
-            skip_ancestor_failed(*mark.args, **mark.kwargs)
-            for mark in ancestor_failed_marks
-        )
-        raise SkippedAncestorFailed(message)
-
     is_skipped = has_mark(task, "skip")
     if is_skipped:
         raise Skipped
@@ -75,6 +68,14 @@ def pytask_execute_task_setup(task: Task) -> None:
         if should_skip:
             raise Skipped(message)
 
+    ancestor_failed_marks = get_marks(task, "skip_ancestor_failed")
+    if ancestor_failed_marks:
+        message = "\n".join(
+            skip_ancestor_failed(*mark.args, **mark.kwargs)
+            for mark in ancestor_failed_marks
+        )
+        raise SkippedAncestorFailed(message)
+
 
 @hookimpl
 def pytask_execute_task_process_report(
@@ -90,11 +91,12 @@ def pytask_execute_task_process_report(
     if report.exc_info:
         if isinstance(report.exc_info[1], SkippedUnchanged):
             report.outcome = TaskOutcome.SKIP_UNCHANGED
+            return True
 
-        elif isinstance(report.exc_info[1], Skipped):
+        if isinstance(report.exc_info[1], Skipped):
             report.outcome = TaskOutcome.SKIP
 
-            for descending_task_name in descending_tasks(task.name, session.dag):
+            for descending_task_name in descending_tasks(task.signature, session.dag):
                 descending_task = session.dag.nodes[descending_task_name]["task"]
                 descending_task.markers.append(
                     Mark(
@@ -103,14 +105,10 @@ def pytask_execute_task_process_report(
                         {"reason": f"Previous task {task.name!r} was skipped."},
                     )
                 )
+            return True
 
-        elif isinstance(report.exc_info[1], SkippedAncestorFailed):
+        if isinstance(report.exc_info[1], SkippedAncestorFailed):
             report.outcome = TaskOutcome.SKIP_PREVIOUS_FAILED
-            report.exc_info = remove_traceback_from_exc_info(report.exc_info)
+            return True
 
-    if report.exc_info and isinstance(
-        report.exc_info[1], (Skipped, SkippedUnchanged, SkippedAncestorFailed)
-    ):
-        return True
-    else:
-        return None
+    return None
